@@ -18,7 +18,7 @@ class Cmake(Xexpect):
         super().__init__(log, exit_nzero_ret=True)
         self.config = config
         self._configure()
-        self._create_mini_sysroot()
+        self._create_app_sysroot()
 
     def _configure(self):
         self.runcmd(f"source {self.config['vitisPath']}/settings64.sh")
@@ -27,11 +27,11 @@ class Cmake(Xexpect):
         f = open(fname, "w")
         f.write(
             f"""
-set( CMAKE_SYSTEM_NAME Linux )                                                 
-set( CMAKE_SYSTEM_PROCESSOR aarch64 )                                          
-set( CMAKE_C_COMPILER aarch64-linux-gnu-gcc )                                  
-set( CMAKE_CXX_COMPILER aarch64-linux-g++ )                                     
-set( CMAKE_ASM_COMPILER aarch64-linux-gnu-gcc )                                
+set( CMAKE_SYSTEM_NAME Linux )
+set( CMAKE_SYSTEM_PROCESSOR aarch64 )
+set( CMAKE_C_COMPILER aarch64-linux-gnu-gcc )
+set( CMAKE_CXX_COMPILER aarch64-linux-g++ )
+set( CMAKE_ASM_COMPILER aarch64-linux-gnu-gcc )
 set( CMAKE_AR aarch64-linux-gnu-ar CACHE FILEPATH "Archiver" )
 set( HUGETLBFS_LIBRARY "{lib_dirs[0]}/libhugetlbfs.so" )
 set( HUGETLBFS_INCLUDE_DIR "{include_dirs[0]}" )
@@ -71,10 +71,10 @@ set( LIBUDEV_INCLUDE_DIR "{include_dirs[0]}" )
 
         return ret
 
-    def _create_mini_sysroot(self):
-        self.mini_sysroot_dir = f"{self.config['workDir']}/mini_sysroot"
-        mkdir(self.mini_sysroot_dir)
-        self.runcmd(f"cd {self.mini_sysroot_dir}")
+    def _create_app_sysroot(self):
+        self.app_sysroot = f"{self.config.workDir}/app_sysroot"
+        mkdir(self.app_sysroot)
+        self.runcmd(f"cd {self.app_sysroot}")
         if self.config.get("rpm_dir"):
             files = get_files(self.config["rpm_dir"], extension="rpm")
             if not files:
@@ -83,10 +83,6 @@ set( LIBUDEV_INCLUDE_DIR "{include_dirs[0]}" )
                 raise Exception(err_msg)
             for rpm in get_files(self.config["rpm_dir"], "rpm"):
                 self.runcmd(f"rpm2cpio {self.config['rpm_dir']}{rpm} | cpio -idmv")
-        self.mini_sysroot_include = f"{self.mini_sysroot_dir}/usr/include"
-        self.mini_sysroot_lib = f"{self.mini_sysroot_dir}/usr/lib"
-        mkdir(f"{self.mini_sysroot_dir}/lib")
-        copyDirectory(f"{self.mini_sysroot_dir}/lib", f"{self.mini_sysroot_lib}")
 
 
 def build_libmetal(cmkobj):
@@ -119,38 +115,64 @@ def build_openamp(cmkobj):
 
 
 def build_aie_lib(cmkobj, component):
+
+    # Initialize variables
+    aiefal_src_basepath = ""
+
+    # export tool chain
     cmd_list = [
-        f'export CC="aarch64-linux-gnu-gcc\
- -I{cmkobj.mini_sysroot_include} -L{cmkobj.mini_sysroot_lib}"',
+        f'export CC="aarch64-linux-gnu-gcc \
+        -I{cmkobj.app_sysroot}/include -L{cmkobj.app_sysroot}/lib"',
         f"cd {cmkobj.config['workDir']}",
     ]
     cmkobj.runcmd_list(cmd_list)
-    if component == "embeddedsw":
-        aie_lib_src = "XilinxProcessorIPLib/drivers/aiengine/src"
-    elif component == "eswv2":
+
+    # AIE driver can be sourced from multiple sources.
+    if component in ("embeddedsw", "eswv2"):
         component = "embeddedsw"
         aie_lib_src = "XilinxProcessorIPLib/drivers/aienginev2/src"
+        aiefal_src_basepath = "XilinxProcessorIPLib/drivers/aiefal/src"
+
+    elif component == "aiert":
+        aie_lib_src = "driver/src"
+        aiefal_src_basepath = "fal/src"
     else:
-        component = "aienginev2"
         aie_lib_src = "src"
 
-    if cmkobj.config[f"external_{component}"]:
-        external_src = cmkobj.config[f"external_{component}"].rstrip("/")
+    # External source is given priority
+    if cmkobj.config[f"external_{component}_src"]:
+        external_src = cmkobj.config[f"external_{component}_src"].rstrip("/")
+        # Copy external source into workdir under component dir
         cmkobj.runcmd(
-            cmd=f"rsync -aqv --exclude '.git*' {external_src} {cmkobj.config.workDir}",
+            cmd=f"rsync -aqv --exclude '.git*' {external_src}/ {cmkobj.config.workDir}/{component}",
             timeout=120,
         )
     else:
+        # Clone the repository into workdir
         clone(
             cmkobj.config.git[f"{component}"],
             os.path.join(cmkobj.config["workDir"], component),
             cmkobj.config["workDir"],
         )
 
+    # Build aie linux library
     aie_lib_src = os.path.join(cmkobj.config["workDir"], component, aie_lib_src)
-    cmd = f"make CFLAGS='-D__AIELINUX__ -Wall' -C {aie_lib_src} -f Makefile.Linux"
-    cmkobj.config["aie_lib_path"] = aie_lib_src
+    aie_lib_build = os.path.join(cmkobj.config["workDir"], f"{component}-build")
+    cmd = f"make CFLAGS='-D__AIELINUX__ -Wall' -C {aie_lib_src} O={aie_lib_build} -f Makefile.Linux"
     cmkobj.runcmd(cmd, expected_failures="Error")
+    # This var is used for deploying sysroot
+    cmkobj.config["aie_lib_path"] = aie_lib_src
+
+    if aiefal_src_basepath:
+        # Get aiefal src path
+        aiefal_src = os.path.join(
+            cmkobj.config["workDir"], component, aiefal_src_basepath
+        )
+        aiefal_include_dir = os.path.join(cmkobj.config["workDir"], "xaiefal")
+        # This var will be used to generate the sysroot providers
+        cmkobj.config["aiefal_include_dir"] = aiefal_include_dir
+        # Build AIEFAL Headers
+        cmkobj.runcmd(f"make -C {aiefal_src} include INCLUDEDIR={aiefal_include_dir}")
 
 
 def aie_linux_lib_builder(config):
@@ -166,23 +188,27 @@ def aie_linux_lib_builder(config):
         ai_engine_src_path, filename="libxaiengine", abs_path=True
     )
 
-    # Copy libs needed for xrt compilation from sysroot
-    libsysroot_files = []
-    for lib in cm.config.get("sysroot_libs", []):
-        libsysroot_files += get_files(
-            f"{config.SYSROOT}/usr/lib", filename=lib, abs_path=True
-        )
-
-    files = libaiengine_files + libsysroot_files
-
     # Deploy generated lib files
-    for lib in files:
+    for lib in libaiengine_files:
         copy_file(lib, f"{config['imagesDir']}")
+
     cmd_list = [
         f"cd {config['imagesDir']}",
         f"tar cvfj {config['test']}.tar.xz ./lib*",
     ]
     cm.runcmd_list(cmd_list)
-    copyDirectory(cm.config["mini_sysroot"], f"{cm.config.imagesDir}/mini_sysroot")
-    ret = True
-    return ret
+
+    # Generate sysroot-provider for aie library
+    # Create dir
+    cm.runcmd("mkdir -p sysroot-provider/include")
+    cm.runcmd("mkdir -p sysroot-provider/lib")
+
+    # Copy lib preserving the symlink to /usr/lib
+    cm.runcmd("cp -Pr ./lib* ./sysroot-provider/lib/")
+    # Copy the headers include dir
+    cm.runcmd(f"cp -Pr {ai_engine_src_path}/../include/* ./sysroot-provider/include/")
+    if config.linux_lib_component in ("embeddedsw", "eswv2", "aiert"):
+        # Copy aiefal include dir
+        cm.runcmd(
+            f"cp -Pr {cm.config['aiefal_include_dir']} ./sysroot-provider/include/"
+        )
