@@ -6,8 +6,8 @@
 from roast.machines.zynqmp import *
 
 dtb_dtg = "zcu216-reva"
-silicon = "zu49-es1"
+silicon = "zu49-prod"
 uboot_devicetree = "zynqmp-zcu216-revA"
 
-PLNX_BSP = "xilinx-zcu216-es1-v{version}-final.bsp"
-plnx_proj = "xilinx-zcu216-es1-{version}"
+PLNX_BSP = "xilinx-zcu216-v{version}-final.bsp"
+plnx_proj = "xilinx-zcu216-{version}"
