@@ -11,21 +11,41 @@ log = logging.getLogger(__name__)
 
 class MtdLinux:
     def is_mtd_exist(self, peripheral):
+        """This Function find mtd nodes and return avaliable node based on peripheral selected"""
+
+        self.console.sync()
+        # To read all mtd node enabled
+        self.console.runcmd(
+            "cat /proc/mtd; echo sync | tr [:lower:] [:upper:]",
+            expected="SYNC",
+            wait_for_prompt=False,
+        )
+        mtdnodes = self.console.output().split("\n")
+        mtdnodes = [i.strip() for i in mtdnodes]
+        mtdlist = list(filter(re.compile("mtd*").match, mtdnodes))
         self.console.sync()
-        self.console.runcmd("cat /proc/mtd", expected="\r\n")
-        items = re.findall("mtd.*:", self.console.output())
-        log.info(f"----- {items} ==")
         self.mtd_list = []
-        for x in items:
-            self.mtd_device = x.split(":")[0]
-            self.mtd_num = int("".join(filter(str.isdigit, self.mtd_device)))
-            self.console.runcmd(
-                f"mtd_debug info /dev/{self.mtd_device}", expected="\r\n"
-            )
-            if ("spi" in peripheral and "MTD_NORFLASH" in self.console.output()) or (
-                "nand" in peripheral and "MTD_NANDFLASH" in self.console.output()
+        # Fetch mtd nodes matched with peripheral
+        for x in mtdlist:
+            if peripheral == "".join(
+                [
+                    i
+                    # Fetch exact peripheral name (spi, qspi, nand etc.)
+                    for i in re.findall(r"\w+\b", re.findall('"(.*?)"', x)[0])[0]
+                    if not i.isdigit()
+                ]
             ):
-                self.mtd_list.append(self.mtd_num)
+                self.mtd_device = x.split(":")[0]
+                self.mtd_num = int("".join(filter(str.isdigit, self.mtd_device)))
+                self.console.runcmd(
+                    f"mtd_debug info /dev/{self.mtd_device}", expected="\r\n"
+                )
+                if (
+                    "spi" in peripheral and "MTD_NORFLASH" in self.console.output()
+                ) or (
+                    "nand" in peripheral and "MTD_NANDFLASH" in self.console.output()
+                ):
+                    self.mtd_list.append(self.mtd_num)
         if not self.mtd_list:
             assert False, f"{peripheral} device is not found"
         return self.mtd_list
@@ -63,9 +83,11 @@ class MtdLinux:
         self.console.runcmd(f"ubidetach --mtdn={mtd_num}")
 
     def flashcptest(self, mtd_num, size):
+        """This function is to perform erase, write, verify data on selected mtd flash node"""
+
         self.console.sync()
         cmdlist = [
-            f"dd if=/dev/urandom of=/tmp/test.img bs={size} count=2",
+            f"dd if=/dev/urandom of=/tmp/test.img bs={size} count=64",
             f"flashcp -v /tmp/test.img /dev/mtd{mtd_num}",
         ]
         self.console.runcmd_list(cmdlist, expected="\r\n")
@@ -76,7 +98,11 @@ class MtdLinux:
         else:
             assert False, "flashcp test failed"
 
-    def mtdmoduletest(self, mtd_num, module, extra_args="", timeout=7200):
+    def mtdmoduletest(
+        self, mtd_num, module, extra_args="", expected="finished", timeout=7200
+    ):
+        """This function is to perform mtd module load test on select mtd flash node"""
+
         self.console.runcmd("lsmod", expected="\r\n")
         if module in self.console.output():
             cmd = f"depmod; rmmod {module}"
@@ -84,11 +110,13 @@ class MtdLinux:
         cmd = f"depmod; modprobe {module} dev={mtd_num} {extra_args}"
         self.console.runcmd(
             cmd,
-            expected="finished",
+            expected=expected,
             expected_failures=[
                 f"{module}: error -22 occurred",
+                "error -28 occurred",
                 "mtd_test: error -22 while erasing EB 0",
                 "spi_master spi0: Failed to power device: -22",
+                "Segmentation fault",
             ],
             timeout=timeout,
         )
